Door = Button:extend()

function Door:new(x, y, doorTexture, attributes)
  Button.super.new(self, x, y, nil, attributes)
  createAttribute(self.attributes, "doorType", doorTexture or "wooden")
  createAttribute(self.attributes, "autoCloseTimer", 10)
  self.type = "door"
  self.isOpen = false
  self.openTimer = 0
  self.openRight = true
  
  --Remove unneeded attributes
  self.attributes.painted = nil
  self.attributes.foreground = nil
  self.attributes.rotation = nil
  
  if not attributes then
    self.attributes.interactRadius = 75
    self.attributes.requiresFacing = true
  end
  
  local doorSet = texture.doors[self.attributes.doorType]
  
  
  self.body = love.physics.newBody(world, x, y, "static")
  self.width = (doorSet.closed:getWidth() - 2 - 8)*smallTextureScale
  self.height = (doorSet.closed:getHeight() - 2)*smallTextureScale
  self.shape = love.physics.newRectangleShape(0, 0, self.width, self.height)
  self.fixture = love.physics.newFixture(self.body, self.shape, 1)
end


function Door:activate(playerPosition)
  if not self.isOpen then
    self:open(playerPosition)
  else
    self:close()
  end
end


function Door:update(dt)
  if self.openTimer ~= 0 then
    if self.attributes.autoCloseTimer > 0 and self.openTimer - dt < 0 then
      self:close()
    else
      self.openTimer = math.max(0, self.openTimer - dt)
    end
  end
end


function Door:open(playerPosition)
  self.openRight = playerPosition.x < 0
  self.isOpen = true
  self.fixture:setFilterData(1,0,0)
  self.openTimer = self.attributes.autoCloseTimer
end


function Door:close()
  self.isOpen = false
  self.openTimer = 0
  self.fixture:setFilterData(1, 65535,0)
end

function Door:draw()
  love.graphics.setColor(1, 1, 1)
  
  local doorSet = texture.doors[self.attributes.doorType]
  
  if self.isOpen then
    if self.openRight then
      blue.graphics.drawCentred(doorSet.open, self.x - doorSet.ox, self.y, 0, -smallTextureScale, smallTextureScale)
    else
      blue.graphics.drawCentred(doorSet.open, self.x + doorSet.ox, self.y, 0, smallTextureScale, smallTextureScale)
    end
  else
    blue.graphics.drawCentred(doorSet.closed, self.x, self.y, 0, smallTextureScale, smallTextureScale)
  end
  
  if debugMode then
    love.graphics.setColor(1, 0, 0)
    blue.graphics.polygon("line", {self.body:getWorldPoints(self.shape:getPoints())})
  end

  if builderMode then
    if self.selected and builderMode then
      love.graphics.setColor(1, 1, 1)
      
      local width = self.width
      local height = self.height

      love.graphics.setLineWidth("3")
      blue.graphics.rectangle("line", self.x - width/2, self.y - height/2, width, height)
      love.graphics.setLineWidth("1")
    end
  end


  if debugMode or (self.selected and builderMode) then
    if self.attributes.interactRadius then
      love.graphics.setColor(1,0,0)
      blue.graphics.circle("line", self.x, self.y, self.attributes.interactRadius)
    end
  end

  self:drawInteractPrompt()
end