Player = Entity:extend()


mouse.primary = false 

local normalFiction = 3


--Class for physics polygons (mainly for drawing them)

function Player:new(x, y)
  Player.super.new(self, x, y)
  self.directionVector = vector.create(1, 0) 
  self.type = "player"
  self.character = "Drea"
  
  self.body = love.physics.newBody(world, x, y, "dynamic")
  self.body:setFixedRotation(true)
  
  -- the player's shape has a radius of 20
  self.shape = love.physics.newCircleShape(20-0.65)
  -- Attach fixture to body and give it a density of 1.
  self.fixture = love.physics.newFixture(self.body, self.shape, 1)
  self.fixture:setRestitution(0.3) -- let the player bounce
  self.fixture:setGroupIndex(-1)
  
  --self.body:setLinearDamping( 0.05 )
  
  self.flightMode = "walking"
  self.tucked = false
  self.timeOfLastFlap = 0
  self.flapping = false
  self.dragVector = vector.create(0,0)
  self.liftVector = vector.create(0,0)
  self.fixture:setFriction(normalFiction)
  self.sleeping = true
  
  self.fixture:setUserData("player")
  self.contactWithGround = false
  self.tangentWithGround = vector.create(-1, 0)
  self.normalWithGround = false
  self.velocityLastFrame = vector.create(0,0)
  
  self.flapCooldown = 8/12
  
  self.tangentVelocity = vector.create(0,0)
  velocityVector = vector.create(0,0)
  
  self.collisionTable = {}
  self.collisionVectorTable = {}
  
  self.bonked = 0
  
  self.stasis = false
  self.stasisCountdown = 0
  self.timeTucked = 0
  
  --self.numberOfSwords = 0
  
  self.textureSet = "drea"
  self.lookDirection = 1
  self.walkRotation = 0
  self.animation = nil
  
  self.animationInfo = {}
  self.animationInfo.walking = nil
  self.animationInfo.lastHitGround = nil
  self.animationInfo.timeOfLastContact = 0
  
  
  self.magic = 0
  self.magicParticleGen = ParticleGen(0,0, 10.0, texture.magic_decay)
end

function Player:update(dt)
  velocityVector = vector.create(self.body:getLinearVelocity()) 
  mouseRelativeToPlayer = vector.create(mouse.worldX - self.body:getX(), mouse.worldY - self.body:getY())
  
  
  self:getCollisionVectors()
  
  if self.character == "Drea" or true then
    self:tuckingBehaviour(dt)
    self:flyingBehaviour(dt)
    self:stasisBehaviour(dt)
    self:boostBehaviour(dt)
    self:bounceParryBehaviour()
    self:keepInBounds()
    
    self.magic = self.magic - dt*0.3
    self.magic = math.max(0, self.magic)
    --self:hoveringBehaviour()
  end
  if self.tucked == false and self.flightMode == "walking" then
    self:walkingBehaviour()
  end
  
  

  
  
  if self.character == "Sift" then
    --[[if clickedThisFrame.primary and self.numberOfSwords < 2 then
      --Note - fix crash when mouse is at 0,0
      self.numberOfSwords = self.numberOfSwords + 1
      local distanceFromPlayer = 50
      local swordPosition = vector.scalarMultiplication(vector.normalize(mouseRelativeToPlayer), 20 + distanceFromPlayer)
      local newSword = Sword(swordPosition.x + self.body:getX(), swordPosition.y + self.body:getY(), mouseRelativeToPlayer)
      
      
      if self.numberOfSwords == 1 then
        self.rope = Rope(self, newSword, 1500)
      elseif self.numberOfSwords == 2 then
        newSword.rope = self.rope
        self.rope:setLength(self.rope.defaultLength)
        self.rope:swapBodies(self.body, newSword.body)
      end
      
    end
    if love.mouse.isDown(2) then
      self.numberOfSwords = 0
    end
    
    self:airWalkingBehaviour()
    ]]
  end 
  
  if controls.lockDirection.thisFrame then
    mouse.storedX = love.mouse.getX()
    mouse.storedY = love.mouse.getY()
  end
  if controls.lockDirection.isDown then
    mouse.storedX = love.mouse.getX()
    mouse.storedY = love.mouse.getY()
  end
  if controls.lockDirection.upThisFrame then
    love.mouse.setPosition(mouse.storedX, mouse.storedY)
  end
  
  
  self:interactBehaviour()
  
  
  if self.contactWithGround then
    self.animationInfo.timeOfLastContact = love.timer.getTime()
  end
  
  
  self.magicParticleGen.rotation = vector.getOrientation(velocityVector) + math.rad(90)
  self.magicParticleGen.x = self.body:getX()
  self.magicParticleGen.y = self.body:getY()
  local minMagic = 0.5
  local maxMagic = 3
  self.magicParticleGen.amount = math.floor(vector.getMagnitudeSqrd(velocityVector)/(160)^2) * math.min(self.magic, maxMagic)/3
  
  
  self.velocityLastFrame = velocityVector
end


function Player:hoveringBehaviour()
  if not self.tucked then
    if controls.hover.thisFrame then

      self.flightMode = toggle(self.flightMode, "hovering", "swooping")
      if self.flightMode == "swooping" then
        self.directionVector = vector.normalize(mouseRelativeToPlayer)
        if self.directionVector.x == 0 and self.directionVector.y == 0 then
          self.directionVector = vector.create(1,0)
        end
      end
      self:untuck()
    end
    if self.flightMode == "hovering" then
      self:hoverControls()
    end
  end
end

function Player:tuckingBehaviour(dt)
  
  --TODO - change this to be based on contact force, not speed so small objects will not bonk you
  if self.contactWithGround and vector.getMagnitude(self.normalVelocity)/64 > 10 and 2 > self.bonked and not self.tucked then
    self.bonked = 1.5
  end
 
  if self.bonked > 0 then
    self:tuck()
    self.bonked = self.bonked - dt
  end
  

  

  

  if self.tucked then
    self.directionVector = vector.angleToVector(self.body:getAngle()) 
    if self.contactWithGround and self.bonked <= 0 then
      if love.keyboard.isScancodeDown("left", "a") then
        --self.body:setAngularVelocity(self.body:getAngularVelocity()+dt*-5)
        self.body:applyTorque(-400)
      end
      if love.keyboard.isScancodeDown("right", "d") then
        --self.body:setAngularVelocity(self.body:getAngularVelocity()+dt*5)
        self.body:applyTorque(400)
      end
    end
  end
  
  
  if controls.tuck.thisFrame or self.bonked > 0 then
    self:tuck()
  elseif controls.tuck.upThisFrame or (self.bonked <= 0 and not controls.tuck.isDown ) then
    self:untuck()
  end 
end

function Player:flyingBehaviour(dt)
  self.isFlapCooldown = love.timer.getTime() - self.timeOfLastFlap <= self.flapCooldown
  if controls.flap.isDown and self.tucked == false then
  
    
    if not self.isFlapCooldown then
      --Set flap timer to current time
      if self.flightMode == "hovering" or self.flightMode == "walking" then
        self.flightMode = "swooping"
        self.directionVector = vector.normalize(mouseRelativeToPlayer)
        if self.directionVector.x == 0 and self.directionVector.y == 0 then
          self.directionVector = vector.create(1,0)
        end
      end
      self:flap()
    end
  end
  
  -- Smooth rotation ==================
  local rotationSpeed = 540 / (vector.getMagnitude(velocityVector)*0.002+1)
  

  if self.flightMode == "swooping" and not self.tucked and not controls.lockDirection.isDown then
    local mouseAngle = vector.getOrientation(mouseRelativeToPlayer)
    local directionAngle = vector.getOrientation(self.directionVector)
    tester = math.deg(mouseAngle - directionAngle)
     --Degrees/second
    local rotationStep = math.rad(rotationSpeed)*dt
    
    --If the difference in angle is less than a rotation step, just set angle to the mouse
    if math.abs(mouseAngle - directionAngle) <= rotationStep then
      self.directionVector = vector.normalize(mouseRelativeToPlayer)
    
    
    elseif vector.angleIsClockwise(self.directionVector, mouseRelativeToPlayer) then      
      --Clockwise
      self.directionVector = vector.angleToVector(directionAngle + rotationStep) 
    else 
      --Anticlockwise 
      self.directionVector = vector.angleToVector(directionAngle - rotationStep) 
    end
  end
  
  -- Aerodynamics area for swooping mode ============================================
  if self.tucked == false and self.flightMode == "swooping" then
    local normal = vector.normalize(vector.create(self.directionVector.y, self.directionVector.x * -1))
    local velocityNormal = vector.normalize(vector.create(velocityVector.y, velocityVector.x * -1))
    --Profile of the line from the velocity vector's position
    local areaVector = vector.project(self.directionVector, velocityNormal)
    --The normal, but with the direction the right direction
    local correctedNormal = vector.normalize(vector.project(velocityVector, normal))
    --Final drag vector
    self.dragVector = vector.scalarMultiplication(correctedNormal, -0.004*vector.getMagnitude(areaVector)*vector.getMagnitudeSqrd(velocityVector))
    self.body:applyForce(self.dragVector.x, self.dragVector.y)
  elseif self.tucked == false and self.flightMode == "hovering" then
    self:walkingBehaviour()
  end
  
  --If moving slow enough, don't bounce
  if vector.getMagnitude(velocityVector)/64 < 10 and not self.tucked then
    self.fixture:setRestitution(0)
  else
    self.fixture:setRestitution(0.3)
  end
    
    
  --Flap wings if valid
  
  local flapRatio = 0.3
  if (love.timer.getTime() - self.timeOfLastFlap  < self.flapCooldown * flapRatio) and self.tucked == false then
    
    self.body:applyForce(self.directionVector.x * 200/flapRatio, self.directionVector.y * 200/flapRatio)
  else
    self.flapping = false
  end
  
  if self.tucked == false and self:canWalk() and not self.flapping then
    self.flightMode = "walking"
  elseif self.flightMode == "swooping" and math.deg(vector.angleBetween(self.directionVector, velocityVector)) > 120 and not self.flapping and vector.getMagnitude(velocityVector)/64 > 3 and (not self.tucked) and not self.stasis and not controls.flap.isDown then
    --Stall if moving too fast backwards
    self.flightMode = "walking"
  end
  
end


function Player:airWalkingBehaviour()
  if not self:canWalk() then
    local maxForce = 25
    
    if love.keyboard.isScancodeDown("right", "d") and love.keyboard.isScancodeDown("left", "a") then
      --Do nothing
    elseif love.keyboard.isScancodeDown("right", "d") then
      self.body:applyForce(maxForce, 0)
      
    elseif love.keyboard.isScancodeDown("left", "a") then
      self.body:applyForce(-maxForce, 0)
    end
  end
end


function Player:stasisBehaviour(dt)
  
  local magicRequirement = 1
  local unfrozeThisFrame = false

  if (self.stasisCountdown <= 0 or controls.stasis.thisFrame or controls.boost.thisFrame) and self.stasis then
    self.stasis = false
    self.body:setType("dynamic")
    self.body:setLinearVelocity(self.directionVector.x*self.storedStasisVelocity, self.directionVector.y*self.storedStasisVelocity)
    self.stasisCountdown = 0
    unfrozeThisFrame = true
  end


  if controls.stasis.thisFrame and (not self.stasis) and (not self.tucked) and self.magic >= magicRequirement and next(self.collisionTable) == nil and not unfrozeThisFrame then
    self.flightMode = "swooping"
    sound.effect.stasis:seek(0)
    sound.effect.stasis:play()
    self.storedStasisVelocity = vector.getMagnitude(vector.create(self.body:getLinearVelocity()))
    self.body:setLinearVelocity(0,0)
    self.stasisCountdown = 1
    self.flapping = false
    self.stasis = true
    self.body:setType("static")
    self.magic = self.magic - magicRequirement
    --Do effect
    TempAnim(self.body:getX(), self.body:getY(), vector.getOrientation(self.directionVector)+math.rad(90), animation.effect.magicRipple)
    
  end
  
  if self.stasisCountdown > 0 then
    self.stasisCountdown = self.stasisCountdown - dt
  end
  

end

function Player:boostBehaviour(dt)
  local magicRequirement = 0.6


  if controls.boost.thisFrame and ((not self.tucked) or (self.bonked > 0)) and self.magic >= magicRequirement then
    local boostAmount = 300
    if self.flightMode == "walking" or self.bonked > 0 then
      
      if self.flightMode == "walking" then
        self.flightMode = "swooping"
      end
      if self.bonked > 0 then
         self.bonked = 0
         self:untuck()
      end
      self.directionVector = vector.normalize(mouseRelativeToPlayer)
    end
    self.magic = self.magic - magicRequirement
    self.body:applyLinearImpulse(self.directionVector.x*boostAmount, self.directionVector.y*boostAmount)
    TempAnim(self.body:getX(), self.body:getY(), vector.getOrientation(self.directionVector)+math.rad(90), animation.effect.magicRipple)
  end
end

function Player:bounceParryBehaviour()
  
  local parryWindow = 0.3
  if self.tucked and (not self.parried) and love.timer.getTime() - self.timeTucked <= parryWindow and next(self.collisionTable) ~= nil and self.bonked <= 0 and self.collisionImpulse >= 300 then
    local sound = sound.effect.bounce_parry:clone()
    sound:play()
    self.parried = true
    self.magic = self.magic + 2
  end
  
end


function Player:interactBehaviour()
  if self.tucked == false and self.flightMode == "walking" then
    local closestInteractable = self:checkInteractables()
    if closestInteractable then
      closestInteractable.nearest = true
    end
    
    if controls.interact.thisFrame then
      self:interact(closestInteractable)
    end
  end
end

function Player:flap()
  sound.effect.wingBeat:play()
  self.sleeping = false
  self.flightMode = "swooping"
  self.timeOfLastFlap = love.timer.getTime()
  self.flapping = true
end


function Player:tuck()
  
  if not self.tucked and not self.stasis then
    self.timeTucked = love.timer.getTime()
    self.fixture:setFriction(0.2)
    self.tucked = true
    self.body:setAngle(vector.getOrientation(self.directionVector))
    self.body:setFixedRotation(false)
    local newRotationSpeed = velocityVector.x/self.shape:getRadius() *1.2 
    if newRotationSpeed > 10 then
      newRotationSpeed = 10
    elseif newRotationSpeed < -10 then
      newRotationSpeed = -10
    end
    
    
    self.body:setAngularVelocity(newRotationSpeed)
  end
end

function Player:untuck()
  
  if self.tucked then
    --cout(self.lookDirection)
    --cout(self.lookDirection)
    self.fixture:setFriction(normalFiction)
    self.tucked = false
    self.parried = false
    if self.flightMode == "swooping" then
      self.directionVector = vector.normalize(mouseRelativeToPlayer)
    else
      self.directionVector = vector.create(self.lookDirection,0)
    end
    self.body:setAngularVelocity(0)
    self.body:setFixedRotation(true)
  end
end

function Player:hoverControls()
  self.dragVector = vector.scalarMultiplication(vector.normalize(velocityVector), -0.4*vector.getMagnitude(velocityVector))
  if not love.keyboard.isScancodeDown("down", "s") then
    self.body:applyForce(0, -gravity * self.body:getMass())
  end
  self.body:applyForce(self.dragVector.x, self.dragVector.y)
  
  local hoverVector = vector.create(0,0)
  
  if love.keyboard.isScancodeDown("right", "d") then
    hoverVector = vector.addition(hoverVector, vector.create(1, 0))
  end
  if love.keyboard.isScancodeDown("left", "a") then
    hoverVector = vector.addition(hoverVector, vector.create(-1, 0))
  end
  if love.keyboard.isScancodeDown("up", "w") and not love.keyboard.isScancodeDown("down", "s") then
    hoverVector = vector.addition(hoverVector, vector.create(0, -1))
  end
  
  hoverVector = vector.normalize(hoverVector)
  local maxForce = 120 + gravity * self.body:getMass()
  local wingForce = hoverFlightFunction(vector.create(0, -gravity * self.body:getMass()), hoverVector, maxForce)
  hoverVector = vector.scalarMultiplication(hoverVector, wingForce)
  
  self.body:applyForce(hoverVector.x, hoverVector.y)
  self.directionVector = vector.normalize(vector.addition(hoverVector, vector.create(0, -gravity*0.5 * self.body:getMass())))
end


function Player:canWalk()
  local maxAngle = math.rad(60)
  if self.contactWithGround then
    return math.acos(vector.dotProduct(self.normalWithGround, vector.create(0,-1))) <= maxAngle
  else 
    return false
  end
end


function Player:canWalkPart(normal)
  local maxAngle = math.rad(60)
  return math.acos(vector.dotProduct(normal, vector.create(0,-1))) <= maxAngle
end



function Player:walkingBehaviour()
  if self:canWalk() then
  --Need to get relative tangential velocity without losing the sign
  --Project on to x axis, then normalize. Then multiply the resulting vector by the magnitude 
  
    local horizontalTangentVector = vector.project(self.tangentVelocity, vector.create(1,0))
          horizontalTangentVector = vector.scalarMultiplication(horizontalTangentVector, vector.getMagnitude(self.tangentVelocity))
    relativeGroundVelocity = vector.scalarMultiplication(vector.project(velocityVector, self.tangentVelocity), 0.00002)
    local contactWithGround = ((180 - math.deg(vector.angleBetween(self.normalWithGround, vector.create(0,-1)))*2))/180
    
    
    local directionForce = vector.normalize(self.tangentWithGround) -- Vector
    if love.keyboard.isScancodeDown("right", "d") and love.keyboard.isScancodeDown("left", "a") then
      self.sleeping = false
      self.animationInfo.walking = false
      if vector.getMagnitude(self.tangentVelocity) > 0.1 then
        self.directionVector = vector.normalize(self.tangentVelocity) 
      end
    elseif love.keyboard.isScancodeDown("right", "d") then
      self.sleeping = false
      self.animationInfo.walking = true
      self.lookDirection = 1
      local walkForce = -1000 * contactWithGround
      self.directionVector = vector.normalize(vector.scalarMultiplication(directionForce, -1)) 
      directionForce = vector.subtraction(vector.scalarMultiplication(directionForce, walkForce), relativeGroundVelocity)
      
    elseif love.keyboard.isScancodeDown("left", "a") then
      self.sleeping = false
      self.animationInfo.walking = true
      self.lookDirection = -1
      local walkForce = 1000 * contactWithGround
      self.directionVector = vector.normalize(directionForce)
      directionForce = vector.subtraction(vector.scalarMultiplication(directionForce, walkForce), relativeGroundVelocity)
      
    else
      self.animationInfo.walking = false 
      
      if vector.getMagnitude(self.tangentVelocity) > 0.1 then
        --self.directionVector = vector.normalize(self.tangentVelocity) Hopefully disabling this fixes issues and breaks nothing
        --Sliding down slope
      else
        self.directionVector = vector.normalize(vector.project(self.directionVector, self.tangentWithGround))
        --Not moving
      end
    end
    
    
    self.body:applyForce(directionForce.x, directionForce.y)
  elseif love.timer.getTime() - self.animationInfo.timeOfLastContact > 0.2 then
    self.animationInfo.walking = false
  end
end



function hoverFlightFunction(g, F, M)
  -- I'm gonna hate myself later for this
  -- Given known vector G, known unit vector F and a known magnitude of the resultant force M.
  -- vector M (which is unknown) must be at full length, and rests on the end of g, and the other on vector tF. tF (t being the magnitude of F) is of unknown length, but the direction is known.
  
  -- Using pythagorous and the quadratic equation, we can find t
  
  --      If you're not me in the future and reading this, sorry - I did most of it on paper
  
  local a = g.x
  local b = g.y
  local c = F.x
  local d = F.y
  
  local m = a*c + b*d
  local n = c^2 + d^2
  
  --Time for the quadratic equation :_)
  
  if 2*n == 0 then
    return 0
  end
  solution = (-(2*m) + math.sqrt((2*m)^2 - 4*n*(a^2 + b^2 - M^2))) / (2*n)
  
  if solution < 0 then
    print("switched sign")
    solution = (-(2*m) - math.sqrt((2*m)^2 - 4*n*(a^2 + b^2 - M^2))) / (2*n)
  end
  return solution
  --IT WORKS AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
end

function Player:draw()
  local sendMagic = math.min(self.magic/5, 1)^0.5
  shader.energy:send("power", sendMagic)
  
  
  
  if math.abs(self.body:getAngularVelocity()/(2*math.pi)) > 10 then
    
    love.graphics.setColor(0.2, 0.2, 1)
  
  elseif self.flapping == false then 
    if self:canWalk() then
      love.graphics.setColor(1, 1, 1)
      else
      love.graphics.setColor(1, 0.2, 0.2)
    end
  else love.graphics.setColor(0.6, 0.6, 1) end
    
  if debugMode then 
    blue.graphics.circle("fill", self.body:getX(), self.body:getY(), self.shape:getRadius())
    love.graphics.setColor(0, 0, 0)
    blue.graphics.circle("line", self.body:getX(), self.body:getY(), self.shape:getRadius())
    
    
    if self.flightMode == "swooping" and self.tucked == false then
      love.graphics.setColor(0, 0.5, 1)
      blue.graphics.line(self.body:getX() + self.directionVector.x*-80, self.body:getY() + self.directionVector.y*-80, self.body:getX() + self.directionVector.x*80, self.body:getY() + self.directionVector.y*80)
    end
    
    
    if self.tucked then
      love.graphics.setColor(0,0,0,0.5)
      blue.graphics.circle("line", self.body:getX(), self.body:getY(), self.shape:getRadius()+5)
    end
    
      
    love.graphics.setColor(0, 0, 0)
    blue.graphics.vectorLine(self.body:getX(), self.body:getY(), self.directionVector, 35)
    
    if self.normalWithGround and self.contactWithGround then
      love.graphics.setColor(0, 0.9, 0.5)
      blue.graphics.vectorLine(self.body:getX(), self.body:getY(), self.normalWithGround, 60)
    end
  end
    
  --Beginning of texture drawing
  --Get scale size, minusing the bleed boarder
  love.graphics.setShader(shader.energy)
  local rot = 0
  if true or self.tucked then
    rot = vector.getOrientation(self.directionVector) + math.rad(90)
  end
  
  
    
  if not (self.tangentWithGround.x == 0 and self.tangentWithGround.y == 0) then
    self.walkRotation = vector.getOrientation(self.tangentWithGround) + math.rad(-90)
  end
  
  
  
  if math.abs(self.body:getAngularVelocity()/(2*math.pi)) > 10 then 
    love.graphics.setColor(0.2, 0.2, 1)
  else
    love.graphics.setColor(1, 1, 1)
  end



  if self.flightMode ~= "walking" and not self.tucked then
    if self.directionVector.x > 0 then
      self.lookDirection = 1
    else 
      self.lookDirection = -1
    end
  end
  
  self.texture = nil
  if self.tucked then
    self.texture = texture.drea_tucked
  elseif self.flightMode == "swooping" then
    self.texture = texture.drea_flying
  else
    if self.sleeping then
      self.texture = texture.drea_sleeping
    else
      self.texture = texture.drea_idle
    end
    love.graphics.setColor(1, 1, 1)
  end

  if self.animationInfo.walking and self.flightMode == "walking" and not self.tucked then 
    self.animation = "running"
    
    animation.drea[self.animation]:play(self.body:getX(), self.body:getY(), self.walkRotation, smallTextureScale*self.lookDirection, smallTextureScale)

      
  elseif self.flightMode == "hovering" and not self.tucked then
    self.animation = "flapping"
    animation.drea[self.animation]:play(self.body:getX(), self.body:getY(), rot, smallTextureScale, smallTextureScale*self.lookDirection)
    love.graphics.setColor(0,0,0)
    blue.graphics.printShadow("Unfinished (Not started) animation :P", window.width/2+50, window.height/2-100)
    love.graphics.setColor(1,1,1)
  elseif self.flightMode == "swooping" and not self.tucked and (self.flapping or self.isFlapCooldown) then
    self.animation = "flapping"
    animation.drea[self.animation]:play(self.body:getX(), self.body:getY(), rot, smallTextureScale, smallTextureScale*self.lookDirection)
  elseif self.flightMode == "swooping" and not self.tucked then
    self.animation = "gliding"
    animation.drea[self.animation]:play(self.body:getX(), self.body:getY(), rot, smallTextureScale, smallTextureScale*self.lookDirection)
  elseif self.tucked then
    
    love.graphics.setColor(1, 1, 1)
    blue.graphics.drawCentredSet(self.texture, self.body:getX(), self.body:getY(), rot, smallTextureScale, smallTextureScale*self.lookDirection, self.texture.ox, self.texture.oy)
  else

    
    love.graphics.setColor(1, 1, 1)
    blue.graphics.drawCentredSet(self.texture, self.body:getX(), self.body:getY(), self.walkRotation, smallTextureScale*self.lookDirection, smallTextureScale, self.texture.ox, self.texture.oy)
  end
  
  love.graphics.setShader()

  for i, animation in pairs(animation.drea) do
    if i ~= self.animation then
      animation:stop()
    end
  end
   
end


function Player:checkInteractables()
  local playerPos = vector.create(self.body:getPosition())
  local closest = nil
  local closestObject = nil
  local priority = false
  for i, object in ipairs(drawTable) do
    if object.isInteractable and object.active ~= false then
      
      if priority then
        if object.attributes.priority then
          if object.attributes.requiresFacing == false or self.directionVector.x * (object.x - playerPos.x) > 0 then
            local distanceVector = vector.subtraction(object, playerPos)
            local newDistance = vector.getMagnitudeSqrd(distanceVector)
            if newDistance < object.attributes.interactRadius^2 then
              
              if not closest then
                closest = newDistance
                closestObject = object
              else
                if newDistance < closest then
                  closest = newDistance
                  closestObject = object
                end
              end
            end
          end
        end
      else
        if object.attributes.requiresFacing == false or self.directionVector.x * (object.x - playerPos.x) > 0 then
          local distanceVector = vector.subtraction(object, playerPos)
          local newDistance = vector.getMagnitudeSqrd(distanceVector)
          if newDistance < object.attributes.interactRadius^2 then
            if not closest then
              closest = newDistance
              closestObject = object
              if object.attributes.priority then
                priority = true
              end
            else
              if object.attributes.priority then
                priority = true
                closest = newDistance
                closestObject = object
              else
                if newDistance < closest then
                  closest = newDistance
                  closestObject = object
                end
              end
            end
          end
        end
      end
    end
  end
  
  if closest then
    return closestObject
  else
    return nil
  end
end


function Player:interact(closestObject)
  local playerPos = vector.create(self.body:getPosition())
  if closestObject then
    local relativePlayerPosition = vector.subtraction(playerPos, closestObject)
    closestObject:interact(relativePlayerPosition)
  end
end


function Player:keepInBounds()
  local x = self.body:getX() 
  local y = self.body:getY() 
  
  if x > 60000 then
    self.body:setX(-30000)
  end
  
  if x < -60000 then
    self.body:setX(30000)
  end
  
  
  
  if y > 15000 then
    self.body:setPosition(0, -10000)
  end
  
end

