local function treatPolygon(pointTable)
  --Set lowest x and y values to 0, and find the offset
  local newPoints = {}
  local lowestX = pointTable[1]
  local lowestY = pointTable[2]
  local highestX = pointTable[1]
  local highestY = pointTable[2]
  
  for i, coord in ipairs(pointTable) do
    if i > 2 then
      if i%2 == 1 then
        --X
        if coord < lowestX then
          lowestX = coord
        elseif coord > highestX then
          highestX = coord
        end
      else
        --Y
        if coord < lowestY then
          lowestY = coord
        elseif coord > highestY then
          highestY = coord
        end
      end
    end
  end
  
  local offsetX = lowestX - lowestX%smallTextureScale
  local offsetY = lowestY - lowestY%smallTextureScale
  
  
  for i, coord in ipairs(pointTable) do
    if i%2 == 1 then
      --X
      table.insert(newPoints, coord - offsetX)
    else
      --Y
      table.insert(newPoints, coord - offsetY)
      
    end
  end
  
  local width = highestX - lowestX
  local height = highestY - lowestY
  return newPoints, offsetX, offsetY, width, height
end




function LevelPolygon:drawFinalised()
  --love.graphics.setBlendMode('alpha', 'premultiplied')
  local brightness = 1
  
  
  if self.attributes.isBackground then  
    brightness = 0.7
  end

  if self.canvasSet then
    
    love.graphics.setColor(brightness, brightness, brightness, 1)
    love.graphics.setBlendMode("alpha", "premultiplied")
    if self.canvasSet.main then
      for i, canvas in ipairs(self.canvasSet.main) do
        
        local set = self.canvasSet
        blue.graphics.draw(canvas.image, 
          self.body:getX() + set.x, 
          self.body:getY() + set.y, 
          self.body:getAngle(), 
          smallTextureScale, smallTextureScale
        ) 
        
      --[[glowCanvas:renderTo(
        function()
          love.graphics.setColor(1,1,1,1)
          love.graphics.setBlendMode("multiply", "premultiplied")
          love.graphics.setShader(shader.prepareToSubtract)
          blue.graphics.draw(canvas.image, 
          self.body:getX() + set.x, 
          self.body:getY() + set.y, 
          self.body:getAngle(), 
          smallTextureScale, smallTextureScale
          ) 
          love.graphics.setShader()
          love.graphics.setBlendMode("alpha")
        end
        )]]
      end
    end
    

    love.graphics.setColor(brightness, brightness, brightness, 0.3)
    if self.canvasSet.far then
      for i, canvas in ipairs(self.canvasSet.far) do
        local set = self.canvasSet
        canvas.image:setFilter("nearest", "nearest")
        love.graphics.setBlendMode("alpha", "alphamultiply")
        blue.graphics.draw(canvas.image, 
          self.body:getX() + set.x, 
          self.body:getY() + set.y, 
          self.body:getAngle(), 
          smallTextureScale, smallTextureScale
        ) 
        love.graphics.setBlendMode('alpha')
        
        
        
      end
    end
    
  end
  love.graphics.setBlendMode("alpha")
end





function LevelPolygon:getCanvasSet()

  local pointTable, x, y, width, height = treatPolygon(self.points)
  
  width  =  width/smallTextureScale
  height = height/smallTextureScale

  if pointTable[1] == pointTable[#pointTable-1] and pointTable[2] == pointTable[#pointTable] then
    table.remove(pointTable, #pointTable)
    table.remove(pointTable, #pointTable)
  end
  
  --Some systems
  local maxSize = love.graphics.getSystemLimits()["texturesize"]

  local xDivisions = math.ceil(width/maxSize)
  local yDivisions = math.ceil(height/maxSize)
  
  local xRemainder = width%maxSize
  local yRemainder = height%maxSize

  local currentWidth  = maxSize
  local currentHeight = maxSize
  
  self.canvasSet = {}
  self.canvasSet.main = {}

  self.canvasSet.x = x
  self.canvasSet.y = y

  self.canvasSet.width  = width
  self.canvasSet.height = height
  
  for i = 0, xDivisions-1 do
    for j = 0, yDivisions-1 do
      
      if i == xDivisions-1 then currentWidth  = xRemainder
      else currentWidth  = maxSize end
      if j == yDivisions-1 then currentHeight = yRemainder
      else currentHeight = maxSize end
      
      if currentWidth >= 1 and currentHeight >= 1 then
        table.insert(self.canvasSet.main, self:getCanvas(i * maxSize, j * maxSize, currentWidth, currentHeight, pointTable))
      else
        self.remove = true
      end
    end
  end
  
  
  self.canvasSet.far = {}
  
  for i = 0, xDivisions-1 do
    for j = 0, yDivisions-1 do
      
      if i == xDivisions-1 then 
        currentWidth = xRemainder
      else 
        currentWidth = maxSize 
      end
      if j == yDivisions-1 then 
        currentHeight = yRemainder
      else 
        currentHeight = maxSize 
      end
      
      if currentWidth >= 1 and currentHeight >= 1 then
        table.insert(self.canvasSet.far, self:getCanvas(i * maxSize, j * maxSize, currentWidth, currentHeight, pointTable, 16))
      else
        self.remove = true
      end
    end
  end
end


function LevelPolygon:getCanvas(offsetX, offsetY, width, height, pointTable, scale)
  
  scale = scale or 1
  
  local canvas = love.graphics.newCanvas( width, height, {mipmaps  = "auto"})
  love.graphics.setCanvas{canvas, stencil=true}
    love.graphics.scale(1/smallTextureScale)
    love.graphics.translate(-offsetX, -offsetY)
    love.graphics.setColor(1,1,1,0)
    love.graphics.rectangle("fill", 1,1, width, height)
    love.graphics.stencil(function()
      love.graphics.setStencilTest("greater", 0)
      
      
      if not love.math.isConvex(pointTable) then
        local triangles = love.math.triangulate(pointTable)
        for i, triangle in ipairs(triangles) do
          love.graphics.polygon("fill", triangle)
        end
      else
        love.graphics.polygon("fill", pointTable)
      end
    end, "replace", 1)
  
    local polyTexture = nil
    if self.attributes.textureName then
      polyTexture = texture[self.attributes.textureName]
    else
      polyTexture = texture.stone
    end

    polyTexture:setWrap("repeat", "repeat")
    love.graphics.setColor(1,1,1)
    local quad = love.graphics.newQuad( self.canvasSet.x, self.canvasSet.y, width*smallTextureScale, height*smallTextureScale, polyTexture:getWidth()*smallTextureScale*scale, polyTexture:getHeight()*smallTextureScale*scale)
    
    if self.attributes.isBackground then
      love.graphics.draw( polyTexture, quad, -100, -100, 0, 1, 1, -100, -100)
    else
      love.graphics.draw( polyTexture, quad, 0, 0, 0, 1, 1, 0, 0)
    end
  
    

    love.graphics.setStencilTest()
    love.graphics.translate(offsetX, offsetY)
    love.graphics.scale(smallTextureScale)
  love.graphics.setCanvas()
  --canvas:generateMipmaps()
  
  local canvasPart = {}
  
  canvasPart.image   = canvas
  canvasPart.image:setFilter("linear", "nearest")
  canvasPart.offsetX = offsetX
  canvasPart.offsetY = offsetY
  canvasPart.width   = width
  canvasPart.height  = height
  return canvasPart
end


