Prop = Entity:extend()

--Class for physics polygons (mainly for drawing them)

function Prop:new(x, y, propTexture, attributes)
  Prop.super.new(self, x, y)
  
  self.type = "prop"
  self.isProp = true --Identifier to recognise children
  
  
  if attributes then
    self.attributes = attributes
  else
    self.attributes = {}
    self.attributes.textureName = propTexture
    self.attributes.draw = true
    self.attributes.rotation = 0
    self.attributes.scaleX = 1
    self.attributes.scaleY = 1
    self.attributes.protected = false
  end
  
  createAttribute(self.attributes, "painted", false)
  createAttribute(self.attributes, "foreground", "Not Implemented")
  createAttribute(self.attributes, "priorityDraw", false)
  

  self.texture = texture[self.attributes.textureName]

  table.insert(drawTable, self)
  
  if self.texture.main then
    self.width = self.texture.main:getWidth()
    self.height = self.texture.main:getHeight()
  else
    self.width = self.texture:getWidth()
    self.height = self.texture:getHeight()
  end
  
  
  
end


function Prop:checkSelect()
  
  return isPointInRectangle( mouse.worldX, mouse.worldY, self.x, self.y, self.width*math.abs(self.attributes.scaleX), self.height*math.abs(self.attributes.scaleY))
end


function Prop:updateBuilder()
  if builder.mode == "prop" or builder.mode == "talkative" or builder.mode == "door" then
    if isPointInRectangle( mouse.worldX, mouse.worldY, self.x, self.y, self.width, self.height) then
      if love.keyboard.isScancodeDown("delete") then
        if not self.attributes.protected then
          self.remove = true
          if self.body then
            self.body:destroy()
          end
        end
      end
    end
  end
  if builder.selectedPolygon == self then
    --Selected but not finalised
    self.selected = true
  end
end



function Prop:draw()
  
  if self.attributes.draw then
    self.texture = texture[self.attributes.textureName]

    if self.texture.main then

      love.graphics.setColor(1,1,1)
      blue.graphics.drawCentredSet(self.texture, self.x, self.y, math.rad(tonumber(self.attributes.rotation)), smallTextureScale*self.attributes.scaleX, smallTextureScale*self.attributes.scaleY)
      
        
    else
      if self.attributes.painted then
        love.graphics.setColor(1,1,1,0.7)
      else
        love.graphics.setColor(1,1,1)
      end
      blue.graphics.drawCentred(self.texture, self.x, self.y, math.rad(tonumber(self.attributes.rotation)), smallTextureScale*self.attributes.scaleX, smallTextureScale*self.attributes.scaleY)
      love.graphics.setColor(1,1,1)
    end
  end
  
  --[[if self.texture.glow then
    bloomCanvas:renderTo( 
    function ()
      love.graphics.clear()
      print(1)
      love.graphics.setColor(1,1,0)

      blue.graphics.drawCentred(self.texture.glow, self.x, self.y, math.rad(self.attributes.rotation), smallTextureScale*self.attributes.scaleX, smallTextureScale*self.attributes.scaleY)
      
    end
  )]]
  
  
  if (builderMode or debugMode) and not self.attributes.draw then
    love.graphics.setColor(1,0.5,0.5)
    blue.graphics.circle("fill", self.x, self.y, 10)
    blue.graphics.circle("line", self.x, self.y, 20)
  end
  
  
  
  
  if builderMode then
    if self.selected and builderMode then
      love.graphics.setColor(1, 1, 1)
      
      local width = nil
      local height = nil
      
      if self.texture.main then
        width = self.texture.main:getWidth()*smallTextureScale*self.attributes.scaleX
        height = self.texture.main:getHeight()*smallTextureScale*self.attributes.scaleY
      else
        width = self.texture:getWidth()*smallTextureScale*self.attributes.scaleX
        height = self.texture:getHeight()*smallTextureScale*self.attributes.scaleY
      end
      love.graphics.setLineWidth("3")
      blue.graphics.rectangle("line", self.x - width/2, self.y - height/2, width, height)
      love.graphics.setLineWidth("1")
    end
  end
  
  if debugMode or (self.selected and builderMode) then
    if self.attributes.interactRadius then
      love.graphics.setColor(1,0,0)
      blue.graphics.circle("line", self.x, self.y, self.attributes.interactRadius)
    end
  end

  self:drawInteractPrompt()
end


function Prop:drawInteractPrompt()
  if self.nearest then
    local offset = 8
    love.graphics.setColor(1,1,1)
    blue.graphics.displayKey(self.x - offset, self.y - offset-30, controls.interact.name, 0.5)
    self.nearest = false
  end
end
