Level = Object:extend()


function Level:new(name)
  self.name = name
  
  self.polygons = {}
end


function Level:deload()
  counter = 1
  
  local deleteTable = copyTable(drawTable)
  for i, v in ipairs(deleteTable) do
    if v.type == "levelPolygon" or v.type == "backgroundPolygon" then
      v:deletePolygon()
      counter = counter + 1
      flushDrawTable()
    elseif v.isProp then
      v.remove = true
      if v.body then
        v.body:destroy()
      end
    elseif v.type == "grass" then
      v.remove = true
    elseif v.type == "firefly" then
      v.remove = true
    end
    fireflyTable = {}
    
  end

end



function Level:saveLevel(filename)
  
  local gatheredObjects = {}
  gatheredObjects.rawPolygons = {}
  gatheredObjects.rawProps = {}
  gatheredObjects.rawTalkatives = {}
  gatheredObjects.rawDoors = {}
  gatheredObjects.rawGrass = {}
  gatheredObjects.fireflies = fireflyTable
  
  for i, v in ipairs(drawTable) do
    if v.type == "levelPolygon" or v.type == "backgroundPolygon" then
      
      local reducedPolygon = {}
      reducedPolygon.x = v.x
      reducedPolygon.y = v.y
      reducedPolygon.points = v.points
      reducedPolygon.attributes = v.attributes
      reducedPolygon.originalX = v.originalX
      reducedPolygon.originalY = v.originalY
      
      table.insert(gatheredObjects.rawPolygons, reducedPolygon)
      
    elseif v.type == "prop" then
      
      local reducedProp = {}
      reducedProp.x = v.x
      reducedProp.y = v.y
      reducedProp.attributes = v.attributes
      table.insert(gatheredObjects.rawProps, reducedProp)
      
    elseif v.type == "talkative" then
      
      local reducedTalkative = {}
      reducedTalkative.x = v.x
      reducedTalkative.y = v.y
      reducedTalkative.attributes = v.attributes
      table.insert(gatheredObjects.rawTalkatives, reducedTalkative)
      
    elseif v.type == "door" then
      
      local reducedDoor = {}
      reducedDoor.x = v.x
      reducedDoor.y = v.y
      reducedDoor.attributes = v.attributes
      table.insert(gatheredObjects.rawDoors, reducedDoor)
      
    elseif v.type == "grass" then
      
      local reducedGrass = {}
      reducedGrass.points = v.points
      reducedGrass.flowerCoverage = v.flowerCoverage
      reducedGrass.seed = v.seed
      table.insert(gatheredObjects.rawGrass, reducedGrass)
    end
    
  end
  
  serialized = lume.serialize(gatheredObjects)

  
  --Save file to appdata
  love.filesystem.createDirectory("levels")
  love.filesystem.createDirectory("levels/autosaves")
    
  local success, message = love.filesystem.write("levels/"..filename..".cq", serialized)
    
  if success then
    return 1
  else
    cout(message)
    return 0
  end
end



function Level:loadLevel(filename, deload, rawFile)
    
  deload = deload or true  
  if deload then
    self:deload()
  end
  
  
  
  local fileData = rawFile or love.filesystem.read("levels/"..filename..".cq")
    
  if not fileData then
    error("Error - filedata could not be read")
  end
  
  
  local loadedObjects = lume.deserialize(fileData)
  
  
  if loadedObjects.fireflies then
    for i, v in ipairs(loadedObjects.fireflies) do
      Firefly(v.x, v.y)
    end
  end
  
  for i, polygon in ipairs(loadedObjects.rawPolygons) do
    local thisPoly = LevelPolygon(polygon.x, polygon.y, polygon.points, polygon.attributes)
    
    if polygon.originalX then
      thisPoly.originalX = polygon.originalX
      thisPoly.originalY = polygon.originalY
    end
    
    thisPoly:setAttributes()
  
    
  end
  
  if loadedObjects.rawProps then
    for i, prop in ipairs(loadedObjects.rawProps) do
      Prop(prop.x, prop.y, nil, prop.attributes)
    end
  end
  if loadedObjects.rawTalkatives then
    for i, talkative in ipairs(loadedObjects.rawTalkatives) do
      Talkative(talkative.x, talkative.y, nil, talkative.attributes)
    end
  end
  if loadedObjects.rawDoors then
    for i, door in ipairs(loadedObjects.rawDoors) do
      Door(door.x, door.y, nil, door.attributes)
    end
  end
  
  if loadedObjects.rawGrass then
    for i, grass in ipairs(loadedObjects.rawGrass) do
      local newGrass = Grass()
      newGrass.points = grass.points
      newGrass.flowerCoverage = grass.flowerCoverage
      newGrass.seed = grass.seed
      newGrass.finalised = true
    end
  end
      
end


function love.filedropped(file)
  
  if builderEnabled then
    file:open("r")
    local data = file:read()
    
    currentLevel:loadLevel(nil, true, data)
  end
end



