--My modifications to the standard graphics library
--Intended purpose is to facilitate custom translations of the whole scene
blue = {}
blue.graphics = {}
local rectWarning = false
local pixelWarning = false
-- Transformation to apply to most items in scene

function blue.standardXTransform(x)
  x = x*camera.zoom + (camera.x*camera.zoom + window.width/2)
  return x
end

function blue.standardYTransform(y)
  y = y*camera.zoom + (camera.y*camera.zoom + window.height/2)
  return y
end


function blue.standardTransform(x, y)
  x = blue.standardXTransform(x)
  y = blue.standardYTransform(y)
  return x, y
end

function blue.graphics.rectangle(mode, x, y, width, height)
  if rectWarning == false then
    print("Warning - rect function is unfinished!")
    rectWarning = true
  end
  x, y = blue.standardTransform(x, y)
  love.graphics.rectangle(mode, x, y, width*camera.zoom, height*camera.zoom)
end


function blue.graphics.circle(mode, x, y, radius)

  x, y = blue.standardTransform(x, y)
  if blue.graphics.testIfOnscreen(x, y, radius*2*camera.zoom, radius*2*camera.zoom) then
    love.graphics.circle(mode, x, y, radius*camera.zoom)
  end
end

function blue.graphics.line(x1, y1, x2, y2)
  x1, y1 = blue.standardTransform(x1, y1)
  x2, y2 = blue.standardTransform(x2, y2)
  
  love.graphics.line(x1, y1, x2, y2)
end

function blue.graphics.parallaxCircle(mode, x, y, z, radius)
  x = ((x + camera.x)*camera.zoom)/z + window.width/2
  y = ((y + camera.y)*camera.zoom)/z + window.height/2
  if blue.graphics.testIfOnscreen(x, y, radius*2*camera.zoom, radius*2*camera.zoom) then
    love.graphics.circle(mode, x, y, radius*camera.zoom)
  end
end

function blue.graphics.parallaxEllipse(mode, x, y, z, radiusX, radiusY)
  x = (x + camera.x)/z + window.width/2
  y = (y + camera.y)/z + window.height/2
  if blue.graphics.testIfOnscreen(x, y, radiusX*2*camera.zoom, radiusY*2*camera.zoom) then
    love.graphics.ellipse(mode, x, y, radiusX*camera.zoom, radiusY*camera.zoom)
  end
end


function blue.graphics.parallaxDraw(drawable, x, y, z, r, sx, sy, ox, oy, kx, ky)
  
  if not drawable then
    error("Expected drawable, got nil!")
  end

  if z == 0 then
    --print("WARNING - Z WAS 0. SETTING TO 1")
    z = 1
  end
  
  x = ((x + camera.x)*camera.zoom)/z + window.width/2
  y = ((y + camera.y)*camera.zoom)/z + window.height/2
  
  local textureWidth = drawable:getWidth()*sx/z
  local textureHeight = drawable:getHeight()*sy/z
  

  if kx or ky then
    print("WARNING! parallaxDraw has not been modified to take sheer into account!")
  end
  if not ox then
    ox = 0
  end
  if not oy then
    oy = 0
  end
  if not sx then
    sx = 1
  end
  if not sy then
    sy = 1
  end
  
  if blue.graphics.testIfOnscreen(((x + ox) + textureWidth/2*camera.zoom), (y + oy) + textureHeight/2*camera.zoom, textureWidth*camera.zoom, textureHeight*camera.zoom) then
    love.graphics.draw(drawable, x, y, r, sx/z*camera.zoom, sy/z*camera.zoom, 0, 0, kx, ky)
  end
end


function blue.graphics.draw(drawable, x, y, r, sx, sy, ox, oy, kx, ky)
  
  if not drawable then
    error("Expected drawable, got nil!")
  end
  
  x, y = blue.standardTransform(x, y)
  ox = ox or 0
  oy = oy or 0
  sx = sx or 1
  sy = sy or 1
  
  local textureWidth = drawable:getWidth()*sx
  local textureHeight = drawable:getHeight()*sy
  
  if blue.graphics.testIfOnscreen(((x + ox) + textureWidth/2*camera.zoom), (y + oy) + textureHeight/2*camera.zoom, textureWidth*camera.zoom, textureHeight*camera.zoom) then
    love.graphics.draw(drawable, x, y, r, sx*camera.zoom, sy*camera.zoom, ox, oy, kx, ky)
  end
end



function blue.graphics.drawCentred(drawable, x, y, r, sx, sy, ox, oy, kx, ky)
  
  x, y = blue.standardTransform(x, y)

  ox = ox or 0
  oy = oy or 0
  sx = sx or 1
  sy = sy or 1


  
  local textureWidth = math.abs(drawable:getWidth()*sx)
  local textureHeight = math.abs(drawable:getHeight()*sy)
  
  local testX = (x + ox - textureWidth/2*camera.zoom) + textureWidth/2*camera.zoom
  local testY = (y + oy - textureHeight/2*camera.zoom) + textureHeight/2*camera.zoom
  local testWidth = textureWidth*camera.zoom+ox*2
  local testHeight = textureHeight*camera.zoom+oy*2
  
  if blue.graphics.testIfOnscreen(testX, testY, testWidth, testHeight) then
    love.graphics.draw(drawable, x, y, r, sx*camera.zoom, sy*camera.zoom, ox + drawable:getWidth()/2, oy + drawable:getHeight()/2, kx, ky)
  end
  
end



function blue.graphics.polygon(mode, pointTable)
  
  if pointTable[1] == pointTable[#pointTable-1] and pointTable[2] == pointTable[#pointTable] then
    table.remove(pointTable, #pointTable)
    table.remove(pointTable, #pointTable)
  end
  
  local newPos = {}
  for i, v in ipairs(pointTable) do
  
    if i%2 == 1 then
      table.insert(newPos, blue.standardXTransform(v))
    else
      table.insert(newPos, blue.standardYTransform(v))
    end
  end
  --Convex correction
  
  if not love.math.isConvex(newPos) and mode == "fill" then
    local triangles = love.math.triangulate(newPos)
    for i, triangle in ipairs(triangles) do
      love.graphics.polygon(mode, triangle)

    end
  else
    love.graphics.polygon(mode, newPos)
  end
end

function blue.graphics.testIfOnscreen(x, y, width, height)
  
  --Important note - tests based on the centre of the rectangle, thus requires adjustment to work with functions that start in the corner
  
  --width = width --* camera.zoom
  --height = height-- *camera.zoom
  
  r, g, b, a = love.graphics.getColor()
  
  if debugMode and false then
    love.graphics.setColor(0,0,0,1)
    love.graphics.rectangle("line", x - width/2, y - height/2, width, height)
    love.graphics.setColor(r, g, b, a)
  end
  
  if window.width < x - width/2 or
  0 > x + width/2 or
  window.height < y - height/2 or
  0 > y + height/2
  then
    return false
  else
    return true
  end
  
end

function blue.graphics.vectorLine(x, y, v, length)
  blue.graphics.line(x, y, x + v.x*length, y + v.y*length)
end



function blue.graphics.print(text, x, y, rot, sx, sy, ox, oy, kx, ky)
  
  x, y = blue.standardTransform(x, y)

  love.graphics.print(text, x+1, y+1, rot, sx*camera.zoom, sy*camera.zoom, ox, oy, kx, ky)

end


function blue.graphics.printShadow(text, x, y, rot, sx, sy, ox, oy, kx, ky)

  r, g, b, a = love.graphics.getColor()
  a = 1
  if r == 0 and g == 0 and b == 0 then
    love.graphics.setColor(1, 1, 1, a*0.5) 
  else
    love.graphics.setColor(0, 0, 0) 
  end
  love.graphics.print(text, x+1, y+1, rot, sx, sy, ox, oy, kx, ky)
  love.graphics.setColor(r, g, b, a) 
  love.graphics.print(text, x, y, rot, sx, sy, ox, oy, kx, ky)
  
end




function blue.graphics:wrappedDraw(drawable, x, y, r, scale)
  
  if not x then
    error("X was not given!")
  elseif not y then
    error("Y was not given!")
  end
  
  r = r or 0
  scale =  scale or 1
  x, y = blue.standardTransform(x, y)
  
  drawable:setWrap( "repeat", "repeat" )
  
  local quad = love.graphics.newQuad( -x, -y, window.width, window.height, drawable:getWidth()*camera.zoom*scale, drawable:getHeight()*camera.zoom*scale )
  love.graphics.draw( drawable, quad, 0, 0, r, 1, 1, 0, 0)
end



function blue.graphics:pixellatedDraw(drawable, x, y, r, scale)
  if not pixelWarning then
    print("Warning! This pixellatedDraw() is not designed to be used outside of testing purposes!")
    pixelWarning = true
  end
  if not drawable then
    error("Expected drawable, got nil!")
  elseif not y then
    error("Y was not given!")
  elseif not x then
    error("X was not given!")
  end
  local canvasSize = 500
  
  local Canvas = love.graphics.newCanvas( canvasSize/textureScale, canvasSize/textureScale )
  love.graphics.setCanvas( Canvas )
    love.graphics.scale(1/textureScale)
    love.graphics.draw(drawable, 0, 0)
    love.graphics.scale(textureScale)
  love.graphics.setCanvas(  )
  
  x, y = blue.standardTransform(x, y)
  ox = ox or 0
  oy = oy or 0
  sx = sx or 1
  sy = sy or 1
  
  local textureWidth = drawable:getWidth()*sx
  local textureHeight = drawable:getHeight()*sy
  
  if blue.graphics.testIfOnscreen(((x + ox) + textureWidth/2*camera.zoom), (y + oy) + textureHeight/2*camera.zoom, textureWidth*camera.zoom, textureHeight*camera.zoom) then
    love.graphics.draw(Canvas, x, y, r, sx*camera.zoom*textureScale, sy*camera.zoom*textureScale, ox, oy, kx, ky)
    love.graphics.rectangle("line", x, y, canvasSize*camera.zoom, canvasSize*camera.zoom)
  end
  
end


function blue.graphics.drawCentredSet(set, x, y, r, sx, sy, ox, oy, kx, ky)
  blue.graphics.drawCentred(set.main, x, y, r, sx, sy, ox, oy, kx, ky)
  glowCanvas:renderTo(
    function()
      love.graphics.setColor(1,1,1,1)
      love.graphics.setBlendMode("multiply", "premultiplied")
      love.graphics.setShader(shader.prepareToSubtract)
      blue.graphics.drawCentred(set.main, x, y, r, sx, sy, ox, oy, kx, ky)
      love.graphics.setShader()
      love.graphics.setBlendMode("alpha")
    end
  )
  
  if set.shadow then
    love.graphics.setColor(0, 0, 0, scene.lightHarshness)
    blue.graphics.drawCentred(set.shadow, x, y, r, sx, sy, ox, oy, kx, ky)
  end
  if set.glow then
    love.graphics.setColor(1,1,1)
    glowCanvas:renderTo(
      function()
        blue.graphics.drawCentred(set.glow, x, y, r, sx, sy, ox, oy, kx, ky)
      end
    )
  end
end


function blue.graphics.triangle(mode, x, y, r, radius, thickness)
  
  x, y = blue.standardTransform(x, y)
  
  if blue.graphics.testIfOnscreen(x, y, radius*2*camera.zoom, radius*2*camera.zoom) then
    
    thickness = thickness or 1
    local originalThickness = love.graphics.getLineWidth()
    
    radius = radius * camera.zoom
    local cosR = math.cos(r)
    local sinR = math.sin(r)
    
    local p1 = vector.create(-1  * radius,  0)
    local p2 = vector.create(0.5 * radius,  0.8660254038 * radius)
    local p3 = vector.create(0.5 * radius, -0.8660254038 * radius)
    
    local function rotate(v, cosR, sinR)
      local returnV = {}
      
      returnV.x = v.x * cosR  -  v.y * sinR
      returnV.y = v.y * cosR  +  v.x * sinR
      
      return returnV
    end
    
    p1 = rotate(p1, cosR, sinR)
    p2 = rotate(p2, cosR, sinR)
    p3 = rotate(p3, cosR, sinR)
    
    local points = {
      p1.x+x, p1.y+y,
      p2.x+x, p2.y+y,
      p3.x+x, p3.y+y}
    
    
    love.graphics.setLineWidth(thickness*camera.zoom)
    
    love.graphics.polygon(mode, points)
    
    love.graphics.setLineWidth(originalThickness)

  end
  
end


function love.graphics.displayKey(x, y, letter, scale)
  
  love.graphics.setColor(1,1,1)
  scale = scale or 1
  
  
  
  
  love.graphics.draw(texture.keypress, x, y, 0, smallTextureScale*scale, smallTextureScale*scale)
  
  love.graphics.setColor(0.8,0.8,0.8)
  love.graphics.print(letter, x + 9*smallTextureScale*scale, y + 3.5*smallTextureScale*scale, 0, smallTextureScale*scale, smallTextureScale*scale)
  
  
end


function blue.graphics.displayKey(x, y, letter, scale)
  
  scale = scale or 1
  blue.graphics.draw(texture.keypress, x, y, 0, smallTextureScale*scale, smallTextureScale*scale)
  
  love.graphics.setColor(0.8,0.8,0.8)
  blue.graphics.print(letter, x + 9*smallTextureScale*scale, y + 3.5*smallTextureScale*scale, 0, smallTextureScale*scale, smallTextureScale*scale)
  
  
end


