--vector maths 2.0
vector = {}

function vector.create(x, y)
  v = {}
  v.x = x
  v.y = y
  return v
end

function vector.turn90Degrees(v)

  nv = {}
  nv.x = v.y
  nv.y = v.x * -1
  return nv
end


function vector.simplify(v)
  --Turns a vector into a table with two variables, but no keys
  nv = {}
  nv[1] = v.x
  nv[2] = v.y
  return nv
end

function vector.project(a, b)
  --Project a onto b
  return vector.scalarMultiplication(b, vector.dotProduct(a, b))
end

function vector.dotProduct(a, b)
  return a.x * b.x + a.y * b.y
end

function vector.addition(a, b)
  newVector = {}
  
  newVector.x = a.x + b.x
  newVector.y = a.y + b.y
  
  return newVector
end

function vector.subtraction(a, b)
  newVector = {}
  newVector.x = a.x - b.x
  newVector.y = a.y - b.y
  
  return newVector
end

function vector.scalarMultiplication(v, s)
  newVector = {}
  newVector.x = v.x * s
  newVector.y = v.y * s
  
  return newVector
end

function vector.scalarDivision(v, s)
  newVector = {}
  newVector.x = v.x / s
  newVector.y = v.y / s
  
  return newVector
end


function vector.getMagnitude(v)
  return math.sqrt(v.x^2 + v.y^2)
end


function vector.getMagnitudeSqrd(v)
  return v.x^2 + v.y^2
end


function vector.normalize(v)
  vectorMagnitune = vector.getMagnitude(v)
  local newV = {}
  if vectorMagnitune ~= 0 then
    newV.x = v.x / vectorMagnitune
    newV.y = v.y / vectorMagnitune
  else
    newV.x = 0
    newV.y = 0
  end
  return newV
end

function vector.getOrientation(v)
  local verticalVector = vector.create(0, 1)
  local tempV = vector.create(v.x, v.y)
  
  if v.x > 0 then
    return math.rad(360) - math.acos(vector.dotProduct(tempV, verticalVector)/vector.getMagnitude(tempV))
  else
    return math.acos(vector.dotProduct(tempV, verticalVector)/vector.getMagnitude(tempV))
  end
end


function vector.angleBetween(v1, v2)
  return math.acos(vector.dotProduct(v1, v2)/(vector.getMagnitude(v1)*vector.getMagnitude(v2)))
end


function vector.angleToVector(angle)
  v = {}
  v.x = math.sin(angle) * -1
  v.y = math.cos(angle) 
  
  return v  
end




function vector.angleIsClockwise(v1, v2)


  -- Net normals to v1 on both sides
  local n1 = vector.normalize(vector.create(v1.y, v1.x*-1))
  local n2 = vector.scalarMultiplication(n1, -1)
  
  -- Get the distance between the mouse and points on the normals
  local d1 = vector.getMagnitudeSqrd(vector.subtraction(v2, n1))
  local d2 = vector.getMagnitudeSqrd(vector.subtraction(v2, n2))
  
  -- Whichever normal is closer
  if d1 > d2 then
    return true
  else
    return false
  end  
end


